/*
 * scamper_addresslist.h
 *
 * $Id: scamper_addresslist.h,v 1.24 2005/03/02 01:51:54 mjl Exp $
 *
 * Copyright (C) 2004 The University of Waikato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __SCAMPER_ADDRESSLIST_H
#define __SCAMPER_ADDRESSLIST_H

typedef struct scamper_source_params
{
  /*
   * these parameters end up in a 'scamper_list' structure with the
   * scamper_trace itself:
   *  id:    an id number assigned by a human.
   *  name:  the name of the list being probed.
   *  descr: a description of the addresses that are stored in this source
   */
  uint32_t  id;
  char     *name;
  char     *descr;

  /* how many addresses per cycle of all sources to read */
  int       priority;

  /* if set, this source is adhoc. */
  int       adhoc;

  /* the output file, if somewhere other than the default output file */
  scamper_outfile_t *sof;

  /*
   * if the source is not adhoc, then we're managing a file.
   *  filename:   the name of the file to read
   *  cycle:      if set, the file should be cycled when the source runs dry
   *  autoreload: if set, the file should be re-loaded at each cycle
   */
  char     *filename;
  int       cycle;
  int       autoreload;

} scamper_source_params_t;

typedef struct scamper_source scamper_source_t;

const char *scamper_source_getname(scamper_source_t *source);

void scamper_addresslist_foreach(void *p,
				 int (*func)(void *p, scamper_source_t *src));

scamper_source_t *scamper_addresslist_addsource(scamper_source_params_t *ssp);

/* get the source that matches the name */
scamper_source_t *scamper_addresslist_getsource(char *name);

/* remove the source from rotation */
int scamper_addresslist_delsource(scamper_source_t *source);

/* mark the cycle point */
int scamper_addresslist_cyclesource(scamper_source_t *source);

/* return the next address to probe */
scamper_task_t *scamper_addresslist_get(void);

/* if the address list is empty, this function will tell us */
int scamper_addresslist_isempty(void);

/* tell the caller if the address list appears to have an address ready */
int scamper_addresslist_isready(void);

/* remove all addresses and sources in the address list */
void scamper_addresslist_empty(void);

/* methods to get addresses into the address list to probe */
int scamper_addresslist_addfile(scamper_source_t *ss, char *listname);
int scamper_addresslist_addarray(scamper_source_t *ss, char **array, int len);
int scamper_addresslist_addaddr(scamper_source_t *ss, char *addr);

/* method to call to release any addresses blocked on a trace */
int scamper_addresslist_unhold(scamper_trace_t *trace);

/* functions to free up the core address list data structures */
int scamper_addresslist_init(void);
void scamper_addresslist_cleanup(void);

#endif /* SCAMPER_ADDRESSLIST_H */
