/*
 * scamper_list.c
 *
 * $Id: scamper_list.c,v 1.5 2005/03/15 20:16:40 mjl Exp $
 *
 * Copyright (C) 2005 The University of Waikato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>

#include <netinet/in.h>

#if defined(__APPLE__)
#include <stdint.h>
#endif

#include <stdlib.h>
#include <string.h>

#if defined(DMALLOC)
#include <dmalloc.h>
#endif

#include "scamper_list.h"

#include "utils.h"

scamper_cycle_t *scamper_cycle_alloc(scamper_list_t *list)
{
  scamper_cycle_t *cycle;

  if(list == NULL)
    {
      return NULL;
    }

  if((cycle = malloc_zero(sizeof(struct scamper_cycle))) == NULL)
    {
      return NULL;
    }

  cycle->refcnt = 1;
  cycle->list = list;
  scamper_list_use(list);

  return cycle;
}

scamper_cycle_t *scamper_cycle_use(scamper_cycle_t *cycle)
{
  cycle->refcnt++;
  return cycle;
}

void scamper_cycle_free(scamper_cycle_t *cycle)
{
  if(--cycle->refcnt == 0)
    {
      if(cycle->list != NULL)
	{
	  scamper_list_free(cycle->list);
	}
      free(cycle);
    }

  return;
}

scamper_list_t *scamper_list_alloc(uint32_t id, char *name, char *descr)
{
  scamper_list_t *list;

  if((list = malloc_zero(sizeof(struct scamper_list))) == NULL)
    {
      return NULL;
    }

  list->id = id;
  list->refcnt = 1;

  if(name != NULL && (list->name = strdup(name)) == NULL)
    {
      goto err;
    }

  if(descr != NULL && (list->descr = strdup(descr)) == NULL)
    {
      goto err;
    }

  return list;

 err:
  if(list->name != NULL) free(list->name);
  if(list->descr != NULL) free(list->descr);
  free(list);
  return NULL;
}

scamper_list_t *scamper_list_use(scamper_list_t *list)
{
  list->refcnt++;
  return list;
}

void scamper_list_free(scamper_list_t *list)
{
  if(--list->refcnt == 0)
    {
      if(list->name != NULL) free(list->name);
      if(list->descr != NULL) free(list->descr);
      free(list);
    }

  return;
}
