/*
 * scamper_list.h
 *
 * $Id: scamper_list.h,v 1.2 2005/03/07 23:09:38 mjl Exp $
 *
 * Copyright (C) 2005 The University of Waikato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __SCAMPER_LIST_H
#define __SCAMPER_LIST_H

/*
 * scamper_list:
 *
 * details regarding a list that was fed into scamper for probing.
 *
 * id:       some ID assigned to identify the list by a person
 * name:     the name assigned to the list
 * descr:    optional free-form text describing the list somehow.
 * refcnt:   a count of references to an instance of this struct
 */
typedef struct scamper_list
{
  uint32_t  id;
  char     *name;
  char     *descr;
  int       refcnt;
  void     *internal;
} scamper_list_t;

/*
 * scamper_cycle:
 *
 * details of the cycle that scamper is currently making over the list.
 *
 * list:    the list the cycle is over.
 * id:      the cycle id.
 * refcnt:  a count of references to an instance of this struct
 */
typedef struct scamper_cycle
{
  scamper_list_t *list;
  uint32_t        id;
  int             refcnt;
} scamper_cycle_t;

/*
 * scamper_[list|cycle]_[alloc|use|free]
 *
 * in order to prevent list and cycle objects from being copied many times
 * for use by trace objects, we use a reference counter in each structure
 * so that it is allocated just the once.
 */
scamper_list_t *scamper_list_alloc(uint32_t id, char *name, char *descr);
scamper_list_t *scamper_list_use(scamper_list_t *list);
void scamper_list_free(scamper_list_t *list);

scamper_cycle_t *scamper_cycle_alloc(scamper_list_t *list);
scamper_cycle_t *scamper_cycle_use(scamper_cycle_t *cycle);
void scamper_cycle_free(scamper_cycle_t *cycle);

#endif /* __SCAMPER_LIST_H */
