/*
 * utils.h
 *
 * $Id: utils.h,v 1.35 2005/05/27 03:36:57 mjl Exp $
 *
 * Copyright (C) 2004-2005 The University of Waikato
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __UTILS_H
#define __UTILS_H

int timeval_cmp(const struct timeval *a, const struct timeval *b);

int64_t timeval_diff_msec(const struct timeval *a, const struct timeval *b);
int64_t timeval_diff_usec(const struct timeval *a, const struct timeval *b);

void timeval_add_msec(struct timeval *tv, const int msec);
void timeval_add_usec(struct timeval *tv, const int64_t usec);
void timeval_add_tv(struct timeval *tv, const struct timeval *add);

void timeval_rtt(struct timeval *rtt,
		 const struct timeval *from, const struct timeval *to);

void timeval_cpy(struct timeval *dst, const struct timeval *src);

int gettimeofday_wrap(struct timeval *tv);
void *malloc_wrap(const size_t size, const char *format, ...);
void *realloc_wrap(void *data, const size_t size, const char *func,
		   const char *format, ...);

#ifndef DMALLOC
void *malloc_zero(const size_t size);
#else
#define malloc_zero(size) memset(malloc(size), 0, size)
#endif

int addr6_cmp(const void *a, const void *b);
int addr4_cmp(const void *a, const void *b);
int addr_cmp(const int af, const void *a, const void *b);
void *addr_dup(const int af, const void *addr);

int sockaddr_compose(struct sockaddr *sa,
		     const int af, const void *addr, const int port);
int sockaddr_len(const struct sockaddr *sa);
struct sockaddr *sockaddr_dup(const struct sockaddr *sa);
void *sockaddr_addr(const struct sockaddr *sa);
void *sockaddr_addr_dup(const struct sockaddr *sa);
char *sockaddr_tostr(const struct sockaddr *sa, char *buf, const size_t len);

int fcntl_set(const int fd, const int flags);
int fcntl_unset(const int fd, const int flags);

char *string_nextword(char *str);
char *string_nullterm(char *str, const char *delim);
int string_isprint(const char *str, const int len);
int string_isnumber(const char *str);
int string_isfloat(const char *str);
int string_tolong(const char *str, long *l);

int read_wrap(const int fd, void *ptr, size_t *rc, const size_t rt);
int write_wrap(const int fd, const void *ptr, size_t *wc, const size_t wt);

#if !defined(__sun__)
int sysctl_wrap(int *mib, int len, void **buf, size_t *size);
#endif

uint16_t in_cksum(const void *buf, const size_t len);

#endif /* __UTILS_H */
